/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_ccs18.c                                                                 *
 *                                                                                              *
 *  Compilers:      CCS                                                                         *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    PICC18 specific functions overloads                                         *
 *                                                                                              *
 *  History:        21.01.2009                                                                  *
 *                  01.04.2009 -    All processor specific definitions moved from osa.c into    *
 *                                  this file                                                   *
 *                                                                                              *
 ************************************************************************************************
 */



#define OS_Init_ProcSpec()      // Empty macro for CCS18










/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/


void _OS_ClrReadySetClrCanContinue (void);



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    _OS_SET_FSR_CUR_TASK_W(OS_CODE_POINTER_SIZE);
    _OS_SavePC();
    _OS_bTaskReady = 1;
    _PIC18_POP();
}

/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler without saving current task return point *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPointNoSave (void)
{
    _PIC18_POP();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReady (void)                                                  *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag and return to the sheduler with seaving   *
 *                  task return point                                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReady (void)
{
    _OS_temp = 0;
    _OS_ClrReadySetClrCanContinue();
}

/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReadySetCanContinue (void)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag, set bCanContinue flag and return to the  *
 *                  sheduler with seaving task return point                     *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReadySetCanContinue (void)
{
    _OS_temp = 0xFF;
    _OS_ClrReadySetClrCanContinue();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClrReadySetClrCanContinue (void)                                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Addition function. Called only by GOTO from:                *
 *                      OS_ClearReady()                                         *
 *                      OS_ClearReadySetCanContinue()                           *
 *                  Stack has pointer of task who called theese services.       *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void _OS_ClrReadySetClrCanContinue (void)
{
    _PIC18_POP();

    OS_Flags.bEventError = 0;
    //_fsr = (_FSR_TYPE)&_OS_CurTask->cTempFSR1;
    
    _OS_SET_FSR_CUR_TASK_W(OS_CODE_POINTER_SIZE);
    _OS_SavePC();

    _OS_bTaskReady = 0;

    #if defined(OS_ENABLE_TTIMER)
        _OS_ClearIndfCanContinue();
        if (_OS_temp & 0x01) _OS_bTaskReady = 1;
    #endif

    _PIC18_POP();
}




















/****************************************************************************************************
 * 
 *       GIEH:GIEL
 * 
 * char OS_DI (void)   
 *
 ****************************************************************************************************/
/****************************************************************************************************
 * 
 *       OS_DI 
 * 
 * void OS_RI (char)   
 *
 ****************************************************************************************************/


char OS_DI (void)
{
    char temp;
    temp = _intcon & 0xC0;
    do {
        _intcon &= 0x7F;
    } while (_intcon & 0x80);
    if (_rcon & 0x80)
    {
        do {
            _intcon &= 0xBF;
        } while (_intcon & 0x40);
    }
    return temp;
}


void OS_RI (char temp)
{
    if (temp & 0x80) _intcon |= 0x80;
    if (_rcon & 0x80)
        if (temp & 0x40) _intcon |= 0x40;
}









 
    
    

/*
 ************************************************************************************************
 *                                                                                              *
 *                         C R I T I C A L   S E C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************
 */
//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------
    
/*
 ********************************************************************************
 *                                                                              *
 *  void OS_EnterCriticalSection (void)                                         *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Enter critical section.                                     *
 *                  This function disables interrupts (with saving current      *
 *                  state) and sets system flag bInCriticalSection              *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_EnterCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_EnterCriticalSection (void)
{
    _OST_INT_TYPE temp;
    temp = OS_DI();
    OS_Flags.bInCriticalSection = 1; 

    OS_Flags.bGIE_CTemp = 0;
    if (temp & 0x80) OS_Flags.bGIE_CTemp = 1;

    OS_Flags.bGIEL_CTemp = 0;
    if (temp & 0x40) OS_Flags.bGIEL_CTemp = 1;
}

/*
 ********************************************************************************
 *                                                                              *
 *   void OS_LeaveCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Leave critical section.                                     *
 *                  This function restore interrupt state from OS_Flag temp bits*
 *                  and clears system flag bInCriticalSection                   *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_LeaveCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_LeaveCriticalSection (void)
{
    char temp;
    OS_Flags.bInCriticalSection = 0; 
    temp = 0;
    if (OS_Flags.bGIE_CTemp) temp |= 0x80;
    if (OS_Flags.bGIEL_CTemp) temp |= 0x40;
    OS_RI(temp);
}

//------------------------------------------------------------------------------
#endif      //     #ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------


















//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMER

/****************************************************************************************************
 * 
 *                                  
 *
 ****************************************************************************************************/
/************************************************************************/
/*                                                                      */
/*                                    */
/* (  )                                         */
/*                                                                      */
/************************************************************************/
    void _OS_InitDelay (OST_TIMER Delay)
    {

        _OS_SET_FSR_CUR_TASK();
        _OS_ClearIndfCanContinue();
        _OS_ClearIndfDelay();

        if (Delay)
        {

            Delay ^= -1;
            Delay ++;
            _OS_CurTask->Timer = Delay;
            
            _OS_SET_FSR_CUR_TASK();
            _OS_SetIndfDelay();
            _OS_SetIndfReady();

        }
    }

//______________________________________________________________________________
//******************************************************************************

#endif  // #ifdef OS_TIMER



    










    
    
    
    
    

    


















    
    
    
    
    
/****************************************************************************************************
 * 
 * 
 *                                      
 *  
 *
 ****************************************************************************************************/
    
/************************************************************************/
/*                                                                      */
/*                     */
/*                                   */
/*                                                                      */
/************************************************************************/

 
void _OS_StopTask (void)
{
    _OS_SET_FSR_CUR_TASK();
    _indf = 0;
}




/************************************************************************/
/*                                                                      */
/*     .                              */
/*  :                                                           */
/*      FSR -                                      */
/*      CARRY - =1 - ,                                                */
/*              =0 -    (  OS_TASKS)    */
/*                                                                      */
/************************************************************************/

//______________________________________________________________________________
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------

bit _OS_CreateTask(_OST_INT_TYPE priority, OST_CODE_POINTER TaskAddr)
{
    OST_TASK_POINTER Task;
    
    OS_Flags.bError = 0;

    #if defined(OS_DISABLE_PRIORITY)
    priority = 0;   //    
    #endif

    Task = (OST_TASK_POINTER)OS_TaskVars;
    _OS_temp = OS_TASKS;
    do {
        if (!Task->State.bEnable)
        {
            ((OST_TASK_STATE*)&priority)->bEnable = 1;
            ((OST_TASK_STATE*)&priority)->bReady = 1;
            

            Task->pTaskPointer = TaskAddr;
            #ifdef OS_ENABLE_TTIMER
                Task->Timer = 0;
            #endif
            Task->cTempFSR1 = 0;
            *((OS_TASKS_BANK char*)&Task->State) = priority;
            _fsr = (_FSR_TYPE)Task;
            //    FSR =    
            OS_Flags.bError = 0;
            return 1;
        
        }
        Task ++;
    } while (_OS_temp--);
    //  ,     
    OS_Flags.bError = 1;
    return 0;
}






























 /******************************************************************************************
 *
 *             P T I M E R '    
 *
 ******************************************************************************************/
#ifdef OS_ENABLE_PTIMERS

/************************************************************************/
/*                                                                      */
/************************************************************************/
void _OS_CreatePTimer (OST_PTIMER *ptimer)
{
    _OS_temp = OS_DI();
    
    _fsr = (_FSR_TYPE) ptimer;
    _postinc = *((NEAR char*)&_OS_PTimers.Flags);   //  bActive,
                                                    //  bTimeout,
                                                    //  bRun
                                                    //  bNextEnable
    _postinc = *((NEAR char*)&_OS_PTimers.Next + 0);
    _postinc = *((NEAR char*)&_OS_PTimers.Next + 1);
    _postinc = 0;
    #if  OS_TIMER_SIZE >= 2
    _postinc = 0;
    #endif
    #if  OS_TIMER_SIZE == 4
    _postinc = 0;
    _postinc = 0;
    #endif


    _OS_PTimers.Next = (OST_PTIMER*)ptimer;
    _OS_PTimers.Flags.bNextEnable = 1;
    
    OS_RI(_OS_temp);
}





/************************************************************************/
/*                                                                      */
/************************************************************************/
void _OS_DeletePTimer (OST_PTIMER *ptimer)
{

    OST_PTIMER *ptcb;
    
    char flags;
    _fsr = (_FSR_TYPE)ptimer;
    _OS_ClearIndfTimerActive;           //   
    flags = _postinc;
    *((char*)&ptcb + 0) = _postinc;
    *((char*)&ptcb + 1) = _postinc;
    _fsr = (_FSR_TYPE)&_OS_PTimers;
REPEAT:
    if (_OS_CheckPostincTimerNextEnable)
    {
        if (_indf == *((char*)&ptimer + 0))
        {
            if (_preinc == *((char*)&ptimer + 1)){
                _OS_temp = OS_DI();
                _postdec = *((char*)&ptcb + 1);
                _postdec = *((char*)&ptcb + 0);
                if (!((OST_PTIMER_FLAGS*)&flags)->bNextEnable) _OS_ClearIndfTimerNextEnable;
                return;
            }
            _fsr--;
        } 
        #asm
            movf    _postinc, 0, 0
            movff   _indf, _fsr + 1
            movwf   _fsr + 0, 0
        #endasm
    }
        
}



#endif  //  OS_ENABLE_PTIMERS


























/****************************************************************************************************
 * 
 *                                /               
 *                  
 *
 * (  )
 *
 ****************************************************************************************************/


//------------------------------------------------------------------------------
void _OS_CheckEvent (_OST_INT_TYPE bEvent)
{
    OS_Flags.bTimeout = 0;
    _OS_SET_FSR_CUR_TASK();
    if (bEvent) {
        if (_OS_bTaskReady) {
            #ifdef OS_ENABLE_TTIMER
                _OS_bTaskDelay = 0;
                _indf &= ~0x10;     // Clear bDelay in task descriptor
            #endif
            return;
        }
        _OS_bTaskReady = 1;
    } else {
        _OS_bTaskReady = 0;
    }
    #ifdef OS_ENABLE_TTIMER
        if (_OS_CheckIndfTimeout() && _OS_CheckIndfCanContinue()) {
            OS_Flags.bTimeout = 1;
            return;
        }
    #endif
    _PIC18_POP();
}


    

        

        
